local mainTexture = "Interface\\AddOns\\HavenLoots\\white.tga";
local mainFont = "Fonts\\FRIZQT__.TTF";
RegisterAddonMessagePrefix("HavenLootsNew");
RegisterAddonMessagePrefix("HavenLootsRoll");
RegisterAddonMessagePrefix("HavenLootsRes");
RegisterAddonMessagePrefix("HavenLootsClear");

--##########################################################################
-- class variables
local itemFrame = {};
local rollWindows = {};
local choiceBoxes = {};
local currentRolls = {};
local hasRolled = {};
local links = {};
local closeButton = OhaiyoClasses_BasicFrame.create();
local cpos = 0;
local colors = {{1, 0, 0, 1}, {1, 1, 0, 1}, {0, 0, 1, 1}, {0, 1, 0, 1}};
local sendType = "RAID";
local isShown = false;

for i = 1, 20 do
	itemFrame[i] = OhaiyoClasses_BasicFrame.create();
	choiceBoxes[i] = {};
	rollWindows[i] = {};
	currentRolls[i] = {};
	hasRolled[i] = false;
	for x = 1, 40 do
		rollWindows[i][x] = {};
		for y = 1, 4 do
			rollWindows[i][x][y] = OhaiyoClasses_BasicFrame.create();
		end
	end

	for x = 1, 4 do
		choiceBoxes[i][x] = OhaiyoClasses_BasicFrame.create();
	end
end

--##########################################################################
-- check message is from master looter
local function IsMasterLooter(name)
	local _, _, ID = GetLootMethod();
	if ID == nil or UnitName("raid" ..ID) ~= name then
		return false;
	end
	return true;
end

--##########################################################################
-- display tooltip
local function ItemButtonUpdate(self, elapsed)
	if(IsModifiedClick('COMPAREITEMS') or (GetCVarBool('alwaysCompareItems') and not IsEquippedItem(self.itemLink))) then
		GameTooltip_ShowCompareItem()
	else
		ShoppingTooltip1:Hide()
		ShoppingTooltip2:Hide()
	end

	if(IsModifiedClick('DRESSUP')) then
		ShowInspectCursor()
	else
		ResetCursor()
	end
end

local function ItemButtonEnter(self)
	GameTooltip:SetOwner(self, 'ANCHOR_TOPRIGHT')
	GameTooltip:SetHyperlink(self.itemLink)
	self:SetScript('OnUpdate', ItemButtonUpdate)
end

local function ItemButtonLeave(self)
	GameTooltip:Hide()
	self:SetScript('OnUpdate', nil)
end

--##########################################################################
-- create loot frame
local function HideRollBoxes(i)
	for x = 1, 4 do
		choiceBoxes[i][x]:Hide();
	end
end

local function ShowRollBoxes(i)
	for x = 1, 4 do
		choiceBoxes[i][x]:Show();
	end
end

local function SendLootRoll(self)
	HideRollBoxes(self.itemNumber);
	hasRolled[self.itemNumber] = true;
	SendAddonMessage("HavenLootsRoll", self.itemNumber .." " ..self.rollType, sendType);
end

local function CreateLootFrame(link, i)
	itemFrame[i]:Show();
	itemFrame[i]:GetFrame().itemLink = link;
	itemFrame[i]:SetText(link);
	if hasRolled[i] ~= true then
		ShowRollBoxes(i);
	end
end

local function ClearFrames()
	closeButton:Hide();
	for i = 1, 20 do
		itemFrame[i]:Hide();
		for x = 1, 4 do
			choiceBoxes[i][x]:Hide();
		end
		for x = 1, 40 do
			for y = 1, 4 do
				rollWindows[i][x][y]:Hide();
			end
		end
	end
end

--##########################################################################
-- frame movement functions
local function MouseDownCloseButton(self, button)
	if button == "RightButton" then
		self:StartMoving();
		self.isMoving = true;
	elseif button == "LeftButton" then
		ClearFrames();
		isShown = false;
	end
end

local function MouseUpCloseButton(self, button)
	if self.isMoving then
		self:StopMovingOrSizing();
		self.isMoving = false;
	end
end

--##########################################################################
-- set up frames
local function CreateChoiceBox(box, parent, text, type, inum)
	box:PositionFrames("BOTTOMLEFT", parent, "TOPLEFT", 100, 30, 0, 0);
	local c = colors[type];
	box:SetTexture(mainTexture, 0, 0, 0, 0.5);
	box:SetBackground(mainTexture, 1, 1, 1, 1);
	box:SetColor(c[1], c[1], c[3], c[4]);
	box:SetFont(mainFont, 15, 1, 1, 1, 1);
	box:SetText(text);
	box:Hide();

	box:GetFrame():EnableMouse(true);
	box:GetFrame().rollType = type;
	box:GetFrame().itemNumber = inum;
	box:GetFrame():RegisterForDrag("LeftButtonDown");
	box:GetFrame():SetScript("OnMouseDown", SendLootRoll);
end

local function SetUpLootFrames()
	for i = 1, 20 do
		local parent = nil;
		if i == 1 then
			parent = closeButton:GetFrame();
		else
			parent = itemFrame[i - 1]:GetFrame();
		end
		itemFrame[i]:PositionFrames("TOPLEFT", parent, "TOPRIGHT", 100, 30, 0, 0);
		itemFrame[i]:SetTexture(mainTexture, 0.2, 0.2, 0.2, 0.5);
		itemFrame[i]:SetBackground(mainTexture, 0, 0, 0, 0.7);
		itemFrame[i]:SetFont(mainFont, 10, 1, 1, 1, 1);
		itemFrame[i]:Hide();

		itemFrame[i]:GetFrame():EnableMouse(true);
		itemFrame[i]:GetFrame():SetScript('OnEnter', ItemButtonEnter);
		itemFrame[i]:GetFrame():SetScript('OnLeave', ItemButtonLeave);

		CreateChoiceBox(choiceBoxes[i][4], itemFrame[i]:GetFrame(), "Main Spec", 4, i);
		choiceBoxes[i][4]:SetColor(0, 1, 0, 1);
		CreateChoiceBox(choiceBoxes[i][3], choiceBoxes[i][4]:GetFrame(), "Off Spec", 3, i);
		choiceBoxes[i][3]:SetColor(0, 0, 1, 1);
		CreateChoiceBox(choiceBoxes[i][2], choiceBoxes[i][3]:GetFrame(), "Alt", 2, i);
		choiceBoxes[i][2]:SetColor(1, 1, 0, 1);
		CreateChoiceBox(choiceBoxes[i][1], choiceBoxes[i][2]:GetFrame(), "Sell", 1, i);
		choiceBoxes[i][1]:SetColor(1, 0, 0, 1);
	end
end

local function SetUpCloseButton()
	closeButton:PositionFrames("TOP", "UIParent", "TOPLEFT", 75, 30, 300, -300);
	closeButton:SetTexture(mainTexture, 0.2, 0.2, 0.2, 0.5);
	closeButton:SetBackground(mainTexture, 0, 0, 0, 0.7);
	closeButton:SetFont(mainFont, 15, 1, 1, 1, 1);
	closeButton:SetText("Close");
	closeButton:Hide();

	closeButton:GetFrame():EnableMouse(true);
	closeButton:GetFrame():RegisterForDrag("LeftButtonDown", "RightButtonDown");
	closeButton:GetFrame():SetMovable(true);
	closeButton:GetFrame():SetScript("OnMouseDown",MouseDownCloseButton);
	closeButton:GetFrame():SetScript("OnMouseUp",MouseUpCloseButton);
end

local function SetUpRollWindows()
	for i = 1, 20 do
		for x = 1, 40 do
			for y = 1, 4 do
				local parent = nil;
				if x == 1 then
					parent = itemFrame[i]:GetFrame();
				else
					parent = rollWindows[i][x - 1][y]:GetFrame();
				end
				local c = colors[y];
				rollWindows[i][x][y]:PositionFrames("TOPLEFT", parent, "BOTTOMLEFT", 100, 20, 0, 0);
				rollWindows[i][x][y]:SetTexture(mainTexture, 0, 0, 0, 0.7);
				rollWindows[i][x][y]:SetFont(mainFont, 10, 1, 1, 1, 1);
				rollWindows[i][x][y]:SetBackground(mainTexture, c[1], c[2], c[3], c[4]);
				rollWindows[i][x][y]:Hide();
			end
		end
	end
end

--##########################################################################
-- add to results table
local function DrawResults()
	for i = 1, 20 do
		if currentRolls[i] ~= nil then
			for x = 1, 40 do
				if currentRolls[i][x] ~= nil then
					local name = currentRolls[i][x].name;
					local rollType = currentRolls[i][x].rollType;
					words = {}
					for word in name:gmatch("([^-]+)") do table.insert(words, word) end
					rollWindows[i][x][rollType]:SetText(words[1] ..": " ..currentRolls[i][x].val - (currentRolls[i][x].rollType * 100));
					rollWindows[i][x][rollType]:Show();
					for y = 1, 4 do
						if y ~= rollType then
							rollWindows[i][x][y]:Hide();
						end
					end
				end
			end
		end
	end
end

local function AddToResults(name, rollType, val, itemNumber)
	for i = 1, table.getn(currentRolls[itemNumber]) do
		if currentRolls[itemNumber][i].val <= val then
			local data = {
				rollType = rollType,
				val = val,
				name = name,
			};
			table.insert(currentRolls[itemNumber], i, data);
			return;
		end
	end
	local data = {
		rollType = rollType,
		val = val,
		name = name,
	};
	table.insert(currentRolls[itemNumber], data);
end

--##########################################################################
-- recieve messages
local getRoll = CreateFrame("FRAME");
getRoll:RegisterEvent("CHAT_MSG_ADDON");
local function GetRoll(self, event, prefix, message, channel, sender)
	local formattedName = {}
	for name in sender:gmatch("([^-]+)") do table.insert(formattedName, name) end
	if IsMasterLooter(formattedName[1]) then
		if prefix == "HavenLootsNew" then
			cpos = cpos + 1;
			closeButton:Show();
			links[cpos] = message;
			for i = 1, cpos do
				CreateLootFrame(links[i], i);
			end
			isShown = true;
			DrawResults();
		end
		if prefix == "HavenLootsRes" then
			words = {}
			for word in message:gmatch("[^%s]+") do table.insert(words, word) end
			AddToResults(words[1], tonumber(words[2]), tonumber(words[3]), tonumber(words[4]));
			if isShown == true then
				DrawResults();
			end
		end
		if prefix == "HavenLootsClear" then
			for i = 1, 20 do
				hasRolled[i] = false;
				currentRolls[i] = {};
			end
			cpos = 0;
			ClearFrames();
		end
	end
end
getRoll:SetScript("OnEvent", GetRoll)

local enterWorld = CreateFrame("FRAME");
enterWorld:RegisterEvent("PLAYER_ENTERING_WORLD");
local function EnterWorld(self, event)
	SetUpCloseButton();
	SetUpRollWindows();
	SetUpLootFrames();
end
enterWorld:SetScript("OnEvent", EnterWorld)

--##########################################################################
-- reopen window
SLASH_HL1 = '/hl';
function SlashCmdList.HL(msg, editbox)
	closeButton:Show();
	for i = 1, cpos do
		CreateLootFrame(links[i], i);
	end
	DrawResults();
	isShown = true;
end

--##########################################################################